function y=chop(x,k)
%
% If 1<k<=length(x), the mulprec number x is
% chopped so that k gytes remain.
% If k>length(x), k-length(x) zeros are added to the right
% ("anti-chopping").
% If k<0, the last |k| gytes are chopped(!); (length(x)-k
% gytes remain; |k| must not exceed length(x)-2, see the code.)
% If k=0, nothing is done.
% If k=1, a clerical error is assumed, and it is treated
%    like k=-1.
% In all these cases the exponent is adjusted so that
% val(y) approx. val(x); see the code below.
%
lex=length(x);
if k>lex,
   y=[x, zeros(1,k-lex)]; y(1)=x(1)+lex-k;
   % k-lex zeros are added to the right; the exponent is
   % accordingly adjusted.
elseif k>1,
   y=x(1:k); y(1)=x(1)+lex-k; % see above
elseif k==1,
   y=x(1: max(2,lex-1));
   y(1)=x(1)+ min(1,lex-2);  %a user's clerical error is
   % assumed and corrected by  the code (the same result
   % as for k=-1).
elseif k<0,
   y= x(1: max(2,k+lex));
   y(1)= x(1)+ min(-k, lex-2); % the last |k| gytes should
   % be chopped away, but y must never be shorter than 2
   % gytes. The exponent is adjusted accordingl
end



