 function [cox, six, eix]=expiold(x,s);  % 010506, 010516, 011029
% Computation of cos(x), sin(x), exp(ix) in mulprec,  x real.
% The method is described in the comments below.
%
global P logP log2  eipi4 pilong pilo4
%  P=10^7; logP=log(P); log2=log(2);
%
%   First seek xbar, such that |xbar|<= pi/4 and
%   exp(ix) is related to exp(i*xbar),see below.
pi4=chop(pilo4,s+1); eipi4=chop(eip4, s+1);
if length(x)==1, xflo=x; x=npr(x);
else             x=nize(x); xflo=flo(x),  ;
end
n=round(2*xflo/pi-1/2);
if abs(xflo)<=pi/4, xbar=x; n=-1/2;    %%
elseif n==0, xbar=nize(sub(x,pi4));    %
else
   tmp=mul(pi4,npr(2*n+1),s+1);
   xbar=nize(sub(x,tmp));              %
end
%
%   Next, compute the number of squarings (m), the argument
% (xm=i*xbar/2^m) in the Taylor expansion, and the number of
% terms (k-1) needed in the Taylor expansion, in order that its
% truncation error becomes bounded by P^(-s). The amount
% of work, which is supposed to be proportional to m+k, is
% (approximately) minimized. ( Often m approx k/1.5)
%    An elementary constrained optimization, where  k!  is
% approximated by (k/e)^k (a simplified Stirling formula), and
% some other simplifications are made, leads to the following
% formulas:

k=1.5*(1/log2+sqrt(log2^(-2) + logP*s/1.5/log2 ));
m= max(0, ceil(k + log(abs(flo(xbar)/k))/log2)),
k=ceil(k),   m2=npr(2^m);  tmp=div(xbar, m2, s+1);
xm=[tmp(1), i*tmp(2:length(tmp))];                                %
% Note that xm is imaginary, while x and xbar are real.
%
%   Now comes a "Horner" evaluation of the truncated Taylor
% expansion for e^(xm).  :
bet=npr(1/gamma(k)); alp=bet;                                     %
% The limited accuracy of  bet  is compensated for later.
for j=1:k-1,
   bet=mul(npr(k-j),bet,s+1);
   tmp=mul(xm,alp,s+1);
   alp= chop( add( tmp, bet), s+1);
end
% The last value of  bet  should be unity, but it isn't, since
% the initial value of  bet  had Matlab precision only. The
% final value of  alp  is therefore divided by  bet , in order
% to yield alp=e^xm :

alp=div(alp,bet,s+1);  % betmi1=sub(bet,[0,1]);                    %%
%
% Next, square  alp  m  times in order to obtain
% alp = (e^xm)^(2^m) = e^(i*xbar) :
for j=1:m,
   tmp=mul(alp,alp,s+1);
   alp=tmp;
end
% alp,                                                            %

% Now alp=e^(i*xbar), normalized to s+1 gytes. By reversion of the
% computation on lines 16-22, we have
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            % Quote from lines 16-22:
            % n=round(2*xflo/pi-1/2);
            % if abs(xflo)<=pi/4, xbar=x, n=-1/2,
            % elseif rem(n,4)==0, xbar=nize(sub(x,pi4)),
            % else tmp=mul(pi4,npr(2*n+1),s+1); xbar=nize(sub(x,tmp)),
            % end
%%%%%%%%%%%%%%%%%%%%%%%%%%%

if n==-1/2, eix=alp;
else turn=i^rem(n,4); eix=mul( mul(alp,eip4,s+1),turn,s+1);
% Note that turn=i^n=exp(2n*pi/4*i). See also lines 19-20.
end

cox=real(eix); six=imag(eix); six(1)=cox(1);

% Recall that the first component is the exponent. THIS IS THE END.