function y=expo(x,s);     % 010504, 011029
% Computation of the exponential function in mulprec;
% x is real. The method is described in the comments
% below. It is inspired by the oldtimers Napier and
% Briggs and others. See H.H.Goldstine, A History of
% Numerical Analysis ..., Springer 1977.

global P logP log2 LP
% P=10^7; logP=log(P); log2=log(2);
%
%   First seek xbar, such that exp(x)=exp(xbar)* P^ex, where ex is an
% integer, and  |xbar|<= LP approx. 8.
%
if length(x)==1, xflo=x; x=npr(x);
else             x=nize(x); xflo=flo(x);                       %
end
ex=round(xflo/logP);                                           %
if ex==0, xbar=x;
else xbar=sub(chop(x,s+1),mul(chop(LP,s+1), ex, s+1));         %
  if s>50, disp('Only 49 gytes reliable in expo'), end         %
end
if abs(flo(xbar))<P^(-(s-1)/2),  % Take care of |xbar|==0 or extremely small.
   alp=add(1,xbar); alp(1)=ex+alp(1); y=alp; return,
end
%
%   Next, compute the number of squarings (m), the argument
% (xm=xbar/2^m) in the Taylor expansion, and the number of
% terms (k-1) needed in the Taylor expansion. They are found
% from an (approximate) minimization of the computing time
% (that -- for given s -- is supposed to be proportional to m+k)
% with the constraint that the Taylor error, propagated by the squarings
% to the error of e^xbar, becomes (almost) bounded by P^(-s).
%  A few simplifications are done in the optimization, e.g., log(k!) is
% approximated by k*(log(k)-1)) ( a simplified Stirling formula).
% The following formulas are obtained for k and m.
% Note that xbar~=0 causes no trouble in the following, because
% this case was taken care of by an exit above. Set
%
Lflox=log(abs(flo(xbar))+1);
k=1/log2+sqrt(max(0, log2^(-2)+ Lflox + logP*s/log2 ));
m= max(0, ceil((k + Lflox - log(k))/log2)),
k=ceil(k),  m2=npr(2^m);
xm=div(xbar, m2, s);                    %
%
%   Now comes a "Horner" evaluation of the truncated Taylor
% expansion for e^xm.  :
bet=npr(1/gamma(k)); alp=bet;
for j=1:k-1,
   bet=mul(npr(k-j),bet,s+1);
   tmp= mul(xm,alp,s+1);
   alp= chop(add(tmp, bet), s+1);
end
%
% The last value of  bet  should have been unity, but it isn't, since
% the initial value of  bet  had only Matlab precision. The
% final value of  alp  is therefore divided by  bet, in order
% to yield alp=e^xm :

alp=div(alp,bet,s+1); % betmi1=sub(bet,1),                             %%

% Finally, square alp  m times in order to obtain
% alp=(e^xm)^(2^m)=e^xbar:

for j=1:m, tmp=mul(alp,alp,s+1); alp=tmp; end

% Now alp=e^xbar.

alp(1)=ex+alp(1);
y=rnize(alp);  


