%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  22. extemp.m  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function y=extemp(x,s);     % 010504
% A slower variant of expo(x,s) that doesn't need the constant LP.
% To be used, e.g., in the computation of more accurate
% values of the global constants L10 and LP.
% Computation of the exponential function in mulprec;
% x is real. The method is described in the comments
% below.

global P logP log2 LP
% P=10^7; logP=log(P); log2=log(2);
% While logP is a float, LP is a 17 gyte mulprec value
% of log(P).  The last gyte is uncertain, and probably
% also the last digits of gyte 16.
%
%LP =[ -15  16 1180956  5095831  9788125  9401827  9054945  3207710 4204014 ...
%   1083223  3295306  7730082  6774146  7361651  9804356  2718808 3911354];
%
%   First seek xbar, such that |xbar|<LP (approx. 8)
% (with high probability), and exp(x)=exp(xbar)* P^ex,
% where ex is an integer.
if length(x)==1, xflo=x; x=npr(x);
else             x=nize(x); xflo=flo(x),
end
%ex=round(xflo/logP);
 ex=0, xbar=x;
%else xbar=sub(x,mul(LP(1:min(s+1,17)),npr(ex),s+1)),
%  if s>16, disp('Only 16 gytes reliable in expo'), end
%end
%if zeromulp(xbar), y=[ex,1]; return, end    % y=P^ex, ex is an integer.
%
% Since |LP-logP| is very small. |xbar|<LP with very high
% probability. The normalization at the end of this program
% will help if this condition does not hold.
%
%   Next, compute the number of squarings (m), the argument
% (xbar/2^m) in the Taylor expansion, and the number of
% terms in the Taylor expansion (k-1), so that its
% truncation error becomes bounded by P^(-s), and the amount
% of work, which is supposed to be proportional to m+k, is
% (approximately) minimized. (In fact m and k are almost
% equal.)
%   An elementary constrained optimization, where  k!  is
% approximated by (k/e)^k (a simplified Stirling formula), and
% some other simplifications are made, leads to the following
% formulas:
%
k=1/log2+sqrt( log2^(-2) + logP*s/log2 );
m= max(0, ceil(k + log(abs(flo(xbar)/k))/log2)),
k=ceil(k), %%
m2=npr(2^m);
xm=div(xbar, m2, s);  % xm=xbar/2^m
%
%   Now comes a "Horner" evaluation of the truncated Taylor
% expansion for e^xm.  :
bet=npr(1/gamma(k)); alp=bet;
for j=1:k-1,
   bet=mul(nprec(k-j),bet,s);
   alp= add(mul(xm,alp,s), bet);
end
%
% The end value of bet should be unity, but it isn't, since
% the initial value of bet had normal precision only. The
% final value of alp is therefore divided by bet, in order
% to yield alp=e^xm :
alp=div(alp,bet,s);
%
% Finally, square alp  m times in order to obtain
% e^xbar=(e^xm)^(2^m) :
for j=1:m, tmp=mul(alp,alp,s); alp=tmp;  end   % change this in expo.m too.
 y=nize(alp);
%
% Now alp=e^xbar.
% alp(1)=ex+alp(1);
%
exflo=exp(xflo); floy=flo(y); relerr=floy/exflo-1;
if abs(relerr)>1e-12,
   x, floy, relerr, error('A big error in expo.'),
end

