function [y,iny]=musqrt(x,s);  % 01-05-16, comments revised 01-11-12.
 % x is a mulprec number or a float.
 % returns y=sqrt(x) (and optionally iny=1/sqrt(x) too) normalized
 % to  s  gytes; s is optional. x may be complex, z is uniquely determined
 % by the initial approximation to 1/sqrt(x) (to almost 16 dec.) provided
 % by the ordinary Matlab arithmetic.
 %   First iny=1/sqrt(x) is computed by the quadratically convergent
 % and division-free iteration formula  z:=z(3/2-(x/2)z^2), which is
 % derived from Newton's method for the equation z^{-2}-x=0.
 % Finaly y=sqrt(x)=x*iny is computed.
 %
 global P
 if nargin==1, s=max(6,length(x)); end
 iter=floor(log(s)/log(2));
 % the number of iterations, estimated from a simple analysis.

 % Initial approximation of 1/sqrt(x):
 if length(x)==1, z0=npc(1/sqrt(x)); x=npc(x);  %if x is a float
 else                                           %if x is a mulprec number
   x=nize(x);
   z0=npc(1/sqrt(flo(x)));
 end

 x2=mul( 0.5, x, s);    %x/2
 % The Newton iterations for computing iny = z0 = 1/sqrt(x):
 for j=1:iter
    zp=mul(z0,z0,s);
    sla=mul(zp,x2,s);
    z0= mul(sub( [-1,3*P/2], sla ),z0, s );
 end
 iny=z0; y=mul(x,z0, s);  %
